from django.utils.deprecation import MiddlewareMixin

from threading import local


class AuthenticatedSignalsMiddleware(MiddlewareMixin):
    thread_data = local()

    def process_request(self, request):
        if hasattr(self.thread_data, 'request'):
            raise Exception("Unknown race condition in authenticated signal middleware!")
        self.thread_data.request = request

    def process_response(self, request, response):
        del self.thread_data.request
        return response

    @classmethod
    def current_request(cls):
        return getattr(cls.thread_data, 'request', None)


def current_request():
    return AuthenticatedSignalsMiddleware.current_request()



from django.db.models.signals import ModelSignal

authenticated_pre_save = ModelSignal(providing_args=["instance", "raw", "using", "update_fields", "request"], use_caching=True)
authenticated_post_save = ModelSignal(providing_args=["instance", "raw", "created", "using", "update_fields", "request"], use_caching=True)
authenticated_save = authenticated_post_save
authenticated_pre_delete = ModelSignal(providing_args=["instance", "using", "request"], use_caching=True)
authenticated_post_delete = ModelSignal(providing_args=["instance", "using", "request"], use_caching=True)
authenticated_delete = authenticated_pre_delete
